#include <ESP8266WiFi.h>
#include <DHTesp.h>
#include "ESP8266TimerInterrupt.h"

// Select a Timer Clock
#define USING_TIM_DIV1                false           // for shortest and most accurate timer
#define USING_TIM_DIV16               false           // for medium time and medium accurate timer
#define USING_TIM_DIV256              true            // for longest timer but least accurate. Default

// Init ESP8266 only and only Timer 1
ESP8266Timer ITimer;
volatile uint32_t lastMillis = 0;

DHTesp dht;
int dhtPin = 13;
int maxTemp = 50;
bool roomMode = true;

int buttonPin = 4;

int wifiLedPin = 5;
int wifiLedState = LOW;

int modeLedPin = 12;
int modeLedState = LOW;

int buzzerPin = 14;
int buzzerState = LOW;
int TIMER_INTERVAL_MS = 2000;

const char* ssid = "Radu's S21";
const char* password = "parola123";

const char* server_room = "https://maker.ifttt.com/trigger/room_burning/with/key/l_I8NDht9KxgL2cQAyELCVnIuhyXMI5VSMqmBNwd247";
const char* server_fridge = "https://maker.ifttt.com/trigger/fridge_broken/with/key/l_I8NDht9KxgL2cQAyELCVnIuhyXMI5VSMqmBNwd247";
bool smsAlreadySent = false;

// Function is executed when the button is pressed (interrupt)
ICACHE_RAM_ATTR void buttonPressed() {
  if (roomMode) {
    // Modul frigider
    modeLedState = HIGH;
    maxTemp = 15;
    roomMode = false;
    smsAlreadySent = false;
  }
  else {
    // Modul camera
    modeLedState = LOW;
    maxTemp = 50;
    smsAlreadySent = false;
    roomMode = true;
  }

  digitalWrite(modeLedPin, modeLedState);
}

// Function that sends the message to the phone
void ifttt_func(const char* serverName, float temp) {
  if(WiFi.status()== WL_CONNECTED){
    WiFiClient client;

    Serial.println("Connecting client to maker.ifttt.com");
    if (client.connect("maker.ifttt.com", 80)) {
      // construct the JSON payload
      String jsonString = "";
      jsonString += "{\"value1\":\"";
      jsonString += temp;
      jsonString += "\",\"value2\":\"";
      jsonString += random(40);
      jsonString += "\",\"value3\":\"";
      jsonString += random(40);
      jsonString += "\"}";
      int jsonLength = jsonString.length();  
      String lenString = String(jsonLength);
  
      // construct the POST request
      String postString = "";
      postString += "POST ";
      postString += serverName;
      postString += " HTTP/1.1\r\n";
      postString += "Host: maker.ifttt.com\r\n";
      postString += "Content-Type: application/json\r\n";
      postString += "Content-Length: ";
      postString += lenString + "\r\n";
      postString += "\r\n";
      postString += jsonString; // combine post request and JSON
      
      client.print(postString);

      // Wait for the response
      while(client.connected() && !client.available())
        delay(1);
      client.stop();
      Serial.println("Message sent");
    }
    else {
      Serial.println("ERROR - unable to connect!!!");
    }
  }
  else {
    Serial.println("WiFi disconnected, could not send message");
  }
}

// smsAlreadySent == true means that the temperature is above the allowed limit.
// It makes the buzzer do BEEEEP (pause) BEEEEEP (pause)
void IRAM_ATTR BuzzerTimer() {  
  if (smsAlreadySent) {
    if (buzzerState == HIGH)
      buzzerState = LOW;
    else
      buzzerState = HIGH;
    digitalWrite(buzzerPin, buzzerState);
  }
  else {
    buzzerState = LOW;
    digitalWrite(buzzerPin, buzzerState);
  }
}

// ================================= SETUP ======================================
void setup() {
  Serial.begin(9600);
  delay(10);
  randomSeed(analogRead(0));

  // Buzzer
  pinMode(buzzerPin, OUTPUT);
  buzzerState = LOW;
  attachInterrupt(digitalPinToInterrupt(buttonPin), buttonPressed, RISING);

  // WiFi led
  pinMode(wifiLedPin, OUTPUT);
  wifiLedState = LOW;
  digitalWrite(wifiLedPin, wifiLedState);

  // Mode led
  pinMode(modeLedPin, OUTPUT);
  modeLedState = LOW;
  digitalWrite(modeLedPin, modeLedState);

  // DHT
  dht.setup(dhtPin, DHTesp::DHT11);

  smsAlreadySent = false;

  // Timer + Interrupt for buzzer
  // Interval in microsecs
  if (ITimer.attachInterruptInterval(TIMER_INTERVAL_MS * 1000, BuzzerTimer)) {
    lastMillis = millis();
    Serial.print(F("Starting ITimer OK, millis() = "));
    Serial.println(lastMillis);
  }
  else
    Serial.println(F("Can't set ITimer correctly. Select another freq. or interval"));

  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
 
  WiFi.begin(ssid, password);
 
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  wifiLedState = HIGH;
  digitalWrite(wifiLedPin, wifiLedState);
}

// ==================================== LOOP ======================================
void loop() {
  float temp = dht.getTemperature();

  if (temp >= maxTemp) {
    if (!smsAlreadySent) {
      smsAlreadySent = true;
      if (roomMode)
        ifttt_func(server_room, temp);
      else
        ifttt_func(server_fridge, temp);
    }
  }
  else if (temp < maxTemp-2)  // Let 2 degrees for that weird spot when temp == maxTemp and it fluctuates
    smsAlreadySent = false;
}
 
